
import numpy as np
import sympy as sym

def dc_ideal(grid,vsc_data):
    '''
    VSC dc working in open loop as a grid former.
    
    '''

    params_dict  = grid.dae['params_dict']
    g_list = grid.dae['g'] 
    y_ini_list = grid.dae['y_ini'] 
    u_ini_dict = grid.dae['u_ini_dict']
    y_run_list = grid.dae['y_run'] 
    u_run_dict = grid.dae['u_run_dict']
    h_dict = grid.dae['h_dict']

    #vscs = [
    #    {'bus':'B1','S_n':100e3,'R':0.01,'X':0.1,'R_n':0.01,'X_n':0.1,'R_ng':0.01,'X_ng':3.0,'K_f':0.1,'T_f':1.0,'K_sec':0.5,'K_delta':0.001},
    #    ]

    #for vsc in vsc_data:
        
    name = vsc_data['bus']

    # inputs
    v_dc = sym.Symbol(f'v_dc_{name}', real=True)
    v_sp,v_sn = sym.symbols(f'V_{name}_0_r,V_{name}_1_r', real=True)
    v_spi,v_sni = sym.symbols(f'V_{name}_0_i,V_{name}_1_i', real=True)

    # parameters
    R_s,R_g = sym.symbols(f'R_{name}_s,R_{name}_g', real=True)

    # dynamical states

    
    # algebraic states
    i_sp,i_sn,v_og = sym.symbols(f'i_vsc_{name}_sp,i_vsc_{name}_sn, v_og_{name}', real=True)

    v_tp = v_dc/2.0
    v_tn = v_dc/2.0

    eq_i_sp = v_og + v_tp - R_s*i_sp - v_sp
    eq_i_sn = v_og - v_tn - R_s*i_sn - v_sn
    eq_v_og = -v_og/R_g - i_sp - i_sn 


    g_list += [eq_i_sp,eq_i_sn,eq_v_og] 

    y_ini_list += [i_sp,i_sn,v_og]
    y_run_list += [i_sp,i_sn,v_og]

    # current injections dc side
    idx_r,idx_i = grid.node2idx(f'{name}','a')
    grid.dae['g'] [idx_r] += -i_sp 
    grid.dae['g'] [idx_i] += v_spi/1e3

    idx_r,idx_i = grid.node2idx(f'{name}','b')
    grid.dae['g'] [idx_r] += -i_sn 
    grid.dae['g'] [idx_i] += v_sni/1e3

    V_1 = vsc_data['U_n']

    u_ini_dict.update({f'v_dc_{name}':V_1})
    u_run_dict.update({f'v_dc_{name}':V_1})

    params_dict.update({f'R_{name}_s':vsc_data['R_s'],f'R_{name}_g':vsc_data['R_g']})

    HS_coi  = 1.0
    omega_coi_i = 1.0

    grid.omega_coi_numerator += omega_coi_i
    grid.omega_coi_denominator += HS_coi



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

from pydae.bmapu.vsgs.leon_gvsg import leon_gvsg
from pydae.bmapu.vsgs.leon_vsg_ll import leon_vsg_ll
from pydae.bmapu.vsgs.uvsg_mid import uvsg_mid
from pydae.bmapu.vsgs.uvsg_high import uvsg_high
from pydae.bmapu.vsgs.olives_vsg import olives_vsg


def add_vsgs(grid):

    for item in grid.data['vsgs']:

        data_dict = item

        bus_name = item['bus']                
        name = create_name(grid, data_dict)
        item['name'] = name
                            
        #if item['type'] == 'wo_vsg': p_W, q_var = wo_vsg(grid,name,bus_name,data_dict)
        #if item['type'] == 'pi_vsg': p_W, q_var = pi_vsg(grid,name,bus_name,data_dict)
        if item['type'] == 'uvsg': p_W, q_var = uvsg_mid(grid,name,bus_name,data_dict)
        if item['type'] == 'uvsg_high': p_W, q_var = uvsg_high(grid,name,bus_name,data_dict)
        if item['type'] == 'vsg_co': p_W, q_var = olives_vsg(grid,name,bus_name,data_dict)
        if item['type'] == 'leon_gvsg': p_W, q_var = leon_gvsg(grid,name,bus_name,data_dict)
        if item['type'] == 'vsg_ll': p_W, q_var = leon_vsg_ll(grid,name,bus_name,data_dict)      

        # grid power injection
        idx_bus = grid.buses_list.index(data_dict['bus']) # get the number of the bus where the syn is connected
        if not 'idx_powers' in grid.buses[idx_bus]: grid.buses[idx_bus].update({'idx_powers':0})
        grid.buses[idx_bus]['idx_powers'] += 1

        S_base = sym.Symbol('S_base', real = True)
        grid.dae['g'][idx_bus*2]   += -p_W/S_base
        grid.dae['g'][idx_bus*2+1] += -q_var/S_base


def create_name(grid,data_dict):

    bus_name = data_dict['bus']
        
    if 'name' in data_dict:
        name = data_dict['name']
    else:
        name = bus_name
        
    for gen_id in range(100):
        if name not in grid.generators_id_list:
            grid.generators_id_list += [name]
            break
        else:
            name = name + f'_{gen_id}'

    return name
        

if __name__ == "__main__":

    import pydae.build_cffi as db
    from pydae.bmapu import bmapu_builder
    from pydae.bmapu.vsgs.vsgs import add_vsgs
    import pydae.build_cffi as db
    import sympy as sym

    data = {
        "sys":{"name":"sys2buses","S_base":100e6, "K_p_agc":0.01,"K_i_agc":0.01},       
        "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
                {"name":"2", "P_W":0.0,"Q_var":0.0,"U_kV":20.0}],
        "lines":[{"bus_j":"1", "bus_k":"2", "X_pu":0.15,"R_pu":0.0, "S_mva":900.0}],
        "vsgs":[
            {"bus":"1","type":"vsg_ll",'S_n':10e6,'F_n':50,'K_delta':0.0,
            'R_v':0.01,'X_v':0.1,'K_p':1.0,'K_i':0.1,'K_g':0.0,'K_q':20.0,
            'T_q':0.1,'K_p_v':1e-6,'K_i_v':1e-6}],
        "genapes":[{"bus":"2","S_n":100e6,"F_n":50.0,"R_v":0.0,"X_v":0.1,"K_delta":0.001,"K_alpha":1.0}],
        }

    grid = bmapu_builder.bmapu(data)

    add_vsgs(grid)



import numpy as np
import sympy as sym

def vsg_lpf(grid,data):
    '''
    VSG based on Low Pass Filter    
    '''

    name = data['bus']
    S_n, U_n = sym.symbols(f'S_n_{name}, U_n_{name}', real=True)
    i_dc, v_dc = sym.symbols(f'i_dc_{name}, v_dc_{name}', real=True)
    p_pos, q_pos = sym.symbols(f'p_pos_{name}, q_pos_{name}', real=True)
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'V_{name}_0_r,V_{name}_1_r,V_{name}_2_r,V_{name}_3_r,v_{name}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i,V_{name}_2_i,V_{name}_3_i,v_{name}_o_i', real=True)

    omega_coi = sym.Symbol('omega_coi', real=True)

    phi_nosat, phi, omega = sym.symbols(f'phi_nosat_{name},phi_{name},omega_{name}', real=True)
    m,m_a,m_b,m_c = sym.symbols(f'm_{name},m_a_{name},m_b_{name},m_c_{name}', real=True)

    H,D = sym.symbols(f'H_{name},D_{name}', real=True)
    K_delta,K_f,K_sec = sym.symbols(f'K_delta_{name},K_f_{name},K_sec_{name}', real=True)
    p_agc = sym.Symbol(f'p_agc', real=True) 
    p_c = sym.Symbol(f'p_c_{name}', real=True) 
    q_ref = sym.Symbol(f'q_ref_{name}', real=True) 
    K_soc,p_soc = sym.symbols(f'K_soc_{name},p_soc_{name}', real=True)
    K_v = sym.Symbol(f'K_v_{name}', real=True)
    phi_v  = sym.Symbol(f'phi_v_{name}', real=True)
    xi_q  = sym.Symbol(f'xi_q_{name}', real=True)
    e_m = sym.Symbol(f'e_m_{name}', real=True) 

    v_sa = v_sa_r +  sym.I*v_sa_i
    s_pos = p_pos + sym.I*q_pos
    i_pos = sym.conjugate(s_pos/(3*v_sa))
    e = e_m*sym.exp(sym.I*phi_v)
    v_t = e #+ sym.I*1e-6*i_pos
    m_cplx = v_t/(400.0/np.sqrt(3))
    m_ref = sym.sqrt(sym.re(m_cplx)**2 + sym.im(m_cplx)**2)*0.715 
    m_phi = sym.atan2(sym.im(m_cplx),sym.re(m_cplx))

    p_m = K_f*S_n*(1 - omega) + K_sec*S_n*p_agc + K_soc*p_soc + p_c
    epsilon_q = q_ref - q_pos

    ## derivatives
    dphi_nosat =  2.0*np.pi*50.0*(omega - omega_coi) - K_delta*phi_nosat - 10*(phi_nosat - phi_v)
    domega = 1/(2*H*S_n)*(p_m - p_pos - D*(omega - 1.0))
    dxi_q = epsilon_q  - 1e-6*xi_q

    grid.dae['f'] += [dphi_nosat, domega, dxi_q]
    grid.dae['x'] += [ phi_nosat,  omega, xi_q]

    # 0.70 - K_v*q_pos/S_n
    grid.dae['g'] += [ m_ref - m]
    grid.dae['g'] += [m - m_a]
    grid.dae['g'] += [m - m_b]
    grid.dae['g'] += [m - m_c]
    grid.dae['g'] += [-phi_v + sym.Piecewise((phi_nosat,(phi_nosat>-2*np.pi)&(phi_nosat<2*np.pi)),(0,True))]
    grid.dae['g'] += [-phi + m_phi]
    grid.dae['g'] += [-e_m + 400.0/np.sqrt(3) - (0.0*epsilon_q/S_n + 0.0*xi_q/S_n) ]
    grid.dae['y_ini'] += [m,m_a,m_b,m_c,phi_v,phi,e_m]
    grid.dae['y_run'] += [m,m_a,m_b,m_c,phi_v,phi,e_m]

    for ph in ['a','b','c']:
        grid.dae['u_ini_dict'].pop(f'm_{ph}_{name}')
        grid.dae['u_run_dict'].pop(f'm_{ph}_{name}')

    grid.dae['u_ini_dict'].pop(f'phi_{name}')
    grid.dae['u_run_dict'].pop(f'phi_{name}')
    
    grid.dae['u_ini_dict'].update({f'p_soc_{name}':0.0})
    grid.dae['u_run_dict'].update({f'p_soc_{name}':0.0})

    grid.dae['u_ini_dict'].update({f'p_c_{name}':0.0})
    grid.dae['u_run_dict'].update({f'p_c_{name}':0.0})

    grid.dae['u_ini_dict'].update({f'q_ref_{name}':0.0})
    grid.dae['u_run_dict'].update({f'q_ref_{name}':0.0})

    grid.dae['h_dict'].update({f'p_m_{name}':p_m})
    grid.dae['h_dict'].update({f'm_phi_{name}':m_phi})
    grid.dae['h_dict'].update({f'm_ref_{name}':m_ref})

    grid.dae['xy_0_dict'].update({f"v_dc_{name}":800})
    grid.dae['xy_0_dict'].update({f"omega_{name}":1.0})
    grid.dae['xy_0_dict'].update({f"e_m_{name}":231})

    for item in ['K_f','H','D','K_delta','K_sec','K_v']:
        grid.dae['params_dict'].update({f'{item}_{name}':data[item]})

    grid.dae['params_dict'].update({f'K_soc_{name}':0.0})

    omega_coi_i = S_n*omega
    HS_coi  = S_n
    

    grid.omega_coi_numerator += omega_coi_i
    grid.omega_coi_denominator += HS_coi  
    
def test():
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db

    grid = urisi('vsg_lpf.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()
    model.ini({'p_load_A2_a':20e3,'q_load_A2_a':-10e3,
               'p_load_A2_b':20e3,'q_load_A2_b':-10e3,
               'p_load_A2_c':20e3,'q_load_A2_c':-10e3,},'xy_0.json')
    
    #model.ini({'p_c_A1':0e3},'xy_0.json')
    
    model.report_u()
    model.report_x()
    model.report_y()
    model.report_z()


if __name__ == '__main__':

    #development()
    test()



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
import numba


def add_lines(self):

    for line in self.lines:

        for ibranch in range(line['N_branches']):

            bus_j = line['bus_j']
            bus_k = line['bus_k']

            if 'bus_j_nodes' in line:
                node_j = line['bus_j_nodes'][ibranch]
            else:
                node_j = ibranch

            if 'bus_k_nodes' in line:
                node_k = line['bus_k_nodes'][ibranch]
            else:
                node_k = ibranch

            idx_j = self.nodes_list.index(f"{bus_j}.{node_j}")
            idx_k = self.nodes_list.index(f"{bus_k}.{node_k}")    

            self.A[self.it_branch+ibranch,idx_j] = 1
            self.A[self.it_branch+ibranch,idx_k] =-1   
            #A[it+1,idx_j] = 1
            #A[it+2,idx_k] = 1   
            
            for icol in range(line['N_branches']):
                if line['sym']:
                    if np.abs(line['Y_primitive'][ibranch,icol]) != 0.0:
                        line_name = f"{bus_j}_{node_j}_{bus_k}_{node_k}_{icol}"
                        g_jk = sym.Symbol(f"g_{line_name}", real=True) 
                        b_jk = sym.Symbol(f"b_{line_name}", real=True) 
                        bs_jk = sym.Symbol(f"bs_{line_name}", real=True) 
                        self.G_primitive[self.it_branch+ibranch,self.it_branch+icol] = g_jk
                        self.B_primitive[self.it_branch+ibranch,self.it_branch+icol] = b_jk
                        self.dae['params_dict'].update({str(g_jk):line['Y_primitive'][ibranch,icol].real})
                        self.dae['params_dict'].update({str(b_jk):line['Y_primitive'][ibranch,icol].imag})

                else:
                    if np.abs(line['Y_primitive'][ibranch,icol]) != 0.0:
                        line_name = f"{bus_j}_{node_j}_{bus_k}_{node_k}_{icol}"
                        g_jk = line['Y_primitive'][ibranch,icol].real 
                        b_jk = line['Y_primitive'][ibranch,icol].imag 
                        bs_jk = sym.Symbol(f"bs_{line_name}", real=True) 
                        self.G_primitive[self.it_branch+ibranch,self.it_branch+icol] = g_jk
                        self.B_primitive[self.it_branch+ibranch,self.it_branch+icol] = b_jk

            #B_primitive[it,ibranch+it] = b_jk
            #B_primitive[it+1,it+1] = bs_jk/2
            #B_primitive[it+2,it+2] = bs_jk/2

        self.it_branch += line['N_branches']



def lines_preprocess(self):

    for line in self.lines:
        
        if 'code' in line:
            if 'X' in self.data['line_codes'][line['code']]:
                R_primitive_matrix = np.array(self.data['line_codes'][line['code']]['R'])
                X_primitive_matrix = np.array(self.data['line_codes'][line['code']]['X'])
                Z_primitive_matrix = R_primitive_matrix + 1j*X_primitive_matrix
                Y_primitive_matrix = np.linalg.inv(Z_primitive_matrix*line['m']/1000)
                line.update({'Y_primitive':Y_primitive_matrix}) 
                
                N_branches = Y_primitive_matrix.shape[0]
                line.update({'N_branches':N_branches}) 
        else:
            if 'X' in line:
                N_branches = 4
                if 'N_branches' in line: N_branches = line['N_branches']
                R_primitive_matrix = np.eye(N_branches)*line['R']
                X_primitive_matrix = np.eye(N_branches)*line['X']
                Z_primitive_matrix = R_primitive_matrix + 1j*X_primitive_matrix
                Y_primitive_matrix = np.linalg.inv(Z_primitive_matrix)
                line.update({'Y_primitive':Y_primitive_matrix}) 
                line.update({'N_branches':N_branches})                 
            if 'X_km' in line:
                N_branches = 4
                if 'km' in line:
                    line.update({'m':line['km']*1000})
                if 'N_branches' in line: N_branches = line['N_branches']
                R_primitive_matrix = np.eye(N_branches)*line['R_km']*line['m']/1000
                X_primitive_matrix = np.eye(N_branches)*line['X_km']*line['m']/1000
                Z_primitive_matrix = R_primitive_matrix + 1j*X_primitive_matrix
                Y_primitive_matrix = np.linalg.inv(Z_primitive_matrix)
                line.update({'Y_primitive':Y_primitive_matrix}) 
                line.update({'N_branches':N_branches})   

        if not 'sym' in line:
            line.update({'sym':False})

        self.N_branches += N_branches

        
def add_line_monitors(self):


    for line in self.lines:

        if 'monitor' in line:
            if line['monitor']:
                            
                bus_j_name = line['bus_j']
                bus_k_name = line['bus_k']
  
                for it in range(line['N_branches']):

                    if 'bus_j_nodes' in line:
                        node_j = line['bus_j_nodes'][it]
                    else:
                        node_j = it

                    if 'bus_k_nodes' in line:
                        node_k = line['bus_k_nodes'][it]
                    else:
                        node_k = it

                    self.dae['h_dict'].update({f"i_l_{bus_j_name}_{node_j}_{bus_k_name}_{node_k}_r" : sym.re(self.I_lines[self.it_branch ++it,0])})
                    self.dae['h_dict'].update({f"i_l_{bus_j_name}_{node_j}_{bus_k_name}_{node_k}_i" : sym.im(self.I_lines[self.it_branch ++it,0])})

        self.it_branch += line['N_branches']


def change_line(model,bus_j,bus_k,data_line_code,length,N_branches=4):
    """
    Change line parameters.

    Parameters
    ----------
    model : pydae object
            pydae model of the grid where the line will be modified
    bus_j : str
            name of the from bus of the line
    bus_k : str
            name of the to bus of the line
    data_line_code : dict 
            dictionary with 2 keys 'R' and 'X' and the respective matrices as list of lists (see example below)

            
    Returns
    -------
    None


    Note
    ----

    Line must be considered *symbolic* ("sym":true) when building the model.

    "lines":[{"bus_j": "A2",  "bus_k": "A3",  "code": "UG1", "m": 100.0,"monitor":true,"sym":true}],
       
                 
    Example
    -------

    data_line_code = {"R":[[ 0.211,  0.049,  0.049,  0.049],
                           [ 0.049,  0.211,  0.049,  0.049],
                           [ 0.049,  0.049,  0.211,  0.049],
                           [ 0.049,  0.049,  0.049,  0.211]],
                      "X":[[ 0.747,  0.673,  0.651,  0.673],
                           [ 0.673,  0.747,  0.673,  0.651],
                           [ 0.651,  0.673,  0.747,  0.673],
                           [ 0.673,  0.651,  0.673,  0.747]], "I_max":430.0}
    
    length = 10
    N_branches = 4
    change_line(model,'A2','A3',data_line_code,length,N_branches)
    
    """
    

    R_primitive_matrix = np.array(data_line_code['R'])
    X_primitive_matrix = np.array(data_line_code['X'])
    Z_primitive_matrix = R_primitive_matrix + 1j*X_primitive_matrix
    Y_primitive_matrix = np.linalg.inv(Z_primitive_matrix*length/1000)

    for ibranch in range(N_branches):

        node_j = ibranch
        node_k = ibranch    

        for icol in range(N_branches):
            line_name = f"{bus_j}_{node_j}_{bus_k}_{node_k}_{icol}"
            g_jk  = f"g_{line_name}" 
            b_jk  = f"b_{line_name}" 
            bs_jk = f"bs_{line_name}"
                           
            model.set_value(g_jk,Y_primitive_matrix[ibranch,icol].real)
            model.set_value(b_jk,Y_primitive_matrix[ibranch,icol].imag)

    return None


def codes_matrix(model,data):
    """
    Generates two numpy arrays, code_g_primitives and code_b_primitives, with N rows each. 
    Each row of code_g_primitives has the flatten real part of primitive admitance matrix while code_b_primitives has the imaginary parts.

    Input
    =====

    Dictionary with line codes and their R and X primitive matrices.

    Returns
    =======

    - code_g_primitives: flatten real part of primitive admitance matrix
    - code_b_primitives: flatten imaginary part of primitive admitance matrix  

    """

    code_y_primitives = [] 

    line_codes = data['line_codes']
    for line_code in line_codes:

        R_primitive_matrix = np.array(line_codes[line_code]['R'])
        X_primitive_matrix = np.array(line_codes[line_code]['X'])
        Z_primitive_matrix = R_primitive_matrix + 1j*X_primitive_matrix
        Y_primitive_matrix = np.linalg.inv(Z_primitive_matrix)
        code_y_primitives += [Y_primitive_matrix.flatten()]
    y_primitives = np.array(code_y_primitives)
    codes_g_primitives = y_primitives.real
    codes_b_primitives =  y_primitives.imag
        
    model.codes_g_primitives = codes_g_primitives
    model.codes_b_primitives = codes_b_primitives

    return codes_g_primitives,codes_b_primitives
    
def lines2p_idx(model,data):
    """

    """

    lines2p_indices_list = []
    for line in data['lines']:
        bus_j = line['bus_j']
        bus_k = line['bus_k']
        line_name = f"{bus_j}_{0}_{bus_k}_{0}_{0}"
        idx = model.params_list.index(f"g_{line_name}")
        N_branches = 4
        lines2p_indices_list += [[idx,N_branches]]
        lines2p_indices = np.array(lines2p_indices_list)
    
    model.lines2p_indices = lines2p_indices

    return lines2p_indices


@numba.njit()
def change_line_fast(p,lines_idxs,line_codes_idx,lengths_km,codes_g_primitives,codes_b_primitives,lines2p_indices):
    for it in range(len(lines_idxs)):
        length_km = lengths_km[it]
        line_code_idx = line_codes_idx[it]
        ini_idx = lines2p_indices[lines_idxs[it],0] 
        end_idx = ini_idx + lines2p_indices[lines_idxs[it],1]**2*2

        p[ini_idx:  end_idx:2] = codes_g_primitives[line_code_idx]/length_km
        p[ini_idx+1:end_idx:2] = codes_b_primitives[line_code_idx]/length_km



# -*- coding: utf-8 -*-
"""
Created on october 2023

@author: jmmauricio
"""
import os
import importlib.util
import pytest
from pydae.bmapu import bmapu_builder
import importlib
import numpy as np

#@pytest.fixture
def test_build():
        

        P_bess = 10e6
        E_bess = 20e6

        data = {
        "system":{"name":"smib","S_base":100e6, "K_p_agc":0.0,"K_i_agc":0.0,"K_xif":1e-6},       
        "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":20.0}],
        "vscs":[{"type":"bess_pq","bus":"1","E_kWh":E_bess/1e3,"S_n":P_bess,"K_delta":0.01}],
        "sources":[{"type":"vsource","bus":"1"}]
        }
        
        os.chdir('./src/pydae/temp')
        grid = bmapu_builder.bmapu(data)
        grid.testing = True
        grid.build('temporal')

 
def test_discharge_charge():

    from pydae.temp import temporal    
    model = temporal.model()
    model.Dt = 1.0

    sigma_ref_ini = 0.5

    model.ini({'sigma_ref_1':sigma_ref_ini},'xy_0.json')
    model.run(0.5*3600,{'p_s_ref_1': 1.0}) # half an hour discharging
    model.run(1.0*3600,{'p_s_ref_1':-1.0}) # half an hour charging
    model.post()

    SoC = model.get_values('sigma_1')

    sigma_ini = SoC[0]
    sigma_lower = np.min(SoC)   
    sigma_end = SoC[-1]

    assert sigma_ini == sigma_ref_ini
    assert sigma_lower == pytest.approx(0.25, abs=0.01)
    assert sigma_end == pytest.approx(0.5, abs=0.01)



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def vsc_l_pq(grid,name,bus_name,data_dict):
    '''

    VSC model with L filter coupling and purely algebraic.
    PQ control is implemented. 

    parameters
    ----------

    S_n: nominal power in VA
    U_n: nominal rms phase to phase voltage in V
    F_n: nominal frequency in Hz
    X_s: coupling reactance in pu (base machine S_n)
    R_s: coupling resistance in pu (base machine S_n)

    inputs
    ------

    p_s_ref: active power reference (pu, S_n base)
    q_s_ref: reactive power reference (pu, S_n base)
    v_dc: dc voltage in pu (when v_dc = 1 and m = 1, v_ac = 1)

    example
    -------

    "vscs": [{"bus":"1","type":"vsc_l_pq","S_n":1e6,"U_n":400.0,"F_n":50.0,"X_s":0.1,"R_s":0.01,"monitor":True}]
    
    '''

    sin = sym.sin
    cos = sym.cos

    ## Common
    ### inputs
    V_s = sym.Symbol(f"V_{bus_name}", real=True)
    theta_s = sym.Symbol(f"theta_{bus_name}", real=True)
    v_dc = sym.Symbol(f"v_dc_{name}", real=True)

    X_s = sym.Symbol(f"X_s_{name}", real=True)
    R_s = sym.Symbol(f"R_s_{name}", real=True)


    ## VSC control 
    ### inputs
    p_s_ref = sym.Symbol(f"p_s_ref_{name}", real=True)
    q_s_ref = sym.Symbol(f"q_s_ref_{name}", real=True)
    i_sa_ref = sym.Symbol(f"i_sa_ref_{name}", real=True)
    i_sr_ref = sym.Symbol(f"i_sr_ref_{name}", real=True)

    ### dynamic states

    ### algebraic states
    i_sd_pq_ref,i_sq_pq_ref = sym.symbols(f'i_sd_pq_ref_{name},i_sq_pq_ref_{name}', real=True)
    i_sd_ar_ref,i_sq_ar_ref = sym.symbols(f'i_sd_ar_ref_{name},i_sq_ar_ref_{name}', real=True)
    v_td_ref,v_tq_ref = sym.symbols(f'v_td_ref_{name},v_tq_ref_{name}', real=True)
    v_lvrt = sym.Symbol(f"v_lvrt_{name}", real=True)
     
    ### parameters
    
    ### auxiliar
    delta = theta_s # ideal PLL
    v_sD = V_s*sin(theta_s)  # v_si   e^(-j)
    v_sQ = V_s*cos(theta_s)  # v_sr
    v_sd = v_sD * cos(delta) - v_sQ * sin(delta)   
    v_sq = v_sD * sin(delta) + v_sQ * cos(delta)

    v_m = sym.sqrt(v_sd**2 + v_sq**2)
    lvrt = sym.Piecewise((0.0,v_m>=v_lvrt),(1.0,v_m<v_lvrt))

    ### dynamic equations            

    ### algebraic equations   
    #g_i_sd_pq_ref  = i_sd_pq_ref*v_sd + i_sq_pq_ref*v_sq - p_s_ref  
    #g_i_sq_pq_ref  =-i_sq_pq_ref*v_sd + i_sd_pq_ref*v_sq - q_s_ref 
    #g_i_sd_ar_ref  = i_sd_ar_ref*v_sd/v_m + i_sq_ar_ref*v_sq/v_m - i_sa_ref  
    #g_i_sq_ar_ref  =-i_sq_ar_ref*v_sd/v_m + i_sd_ar_ref*v_sq/v_m - i_sr_ref 
    #g_v_td_ref  = v_td_ref - R_s*i_sd_ref + X_s*i_sq_ref - v_sd  
    #g_v_tq_ref  = v_tq_ref - R_s*i_sq_ref - X_s*i_sd_ref - v_sq 

    i_sd_ar_ref = i_sa_ref*v_sd/sym.sqrt(v_sd**2 + v_sq**2) + i_sr_ref*v_sq/sym.sqrt(v_sd**2 + v_sq**2) 
    i_sq_ar_ref = i_sa_ref*v_sq/sym.sqrt(v_sd**2 + v_sq**2) - i_sr_ref*v_sd/sym.sqrt(v_sd**2 + v_sq**2)

    i_sd_pq_ref = (p_s_ref*v_sd + q_s_ref*v_sq)/(v_sd**2 + v_sq**2)
    i_sq_pq_ref = (p_s_ref*v_sq - q_s_ref*v_sd)/(v_sd**2 + v_sq**2)
    i_sd_ref = (1.0-lvrt)*i_sd_pq_ref + lvrt*i_sd_ar_ref
    i_sq_ref = (1.0-lvrt)*i_sq_pq_ref + lvrt*i_sq_ar_ref

    v_td_ref  =  R_s*i_sd_ref - X_s*i_sq_ref + v_sd  
    v_tq_ref  =  R_s*i_sq_ref + X_s*i_sd_ref + v_sq 

    v_tD_ref = v_td_ref * cos(delta) + v_tq_ref * sin(delta)   
    v_tQ_ref =-v_td_ref * sin(delta) + v_tq_ref * cos(delta)    
    v_ti_ref = v_tD_ref
    v_tr_ref = v_tQ_ref 
    m_ref = sym.sqrt(v_tr_ref**2 + v_ti_ref**2)/v_dc
    theta_t_ref = sym.atan2(v_ti_ref,v_tr_ref) 


    ### dae 
    grid.dae['g'] +=     []
    grid.dae['y_ini'] += []  
    grid.dae['y_run'] += []  
    
    grid.dae['u_ini_dict'].update({f'{str(p_s_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(p_s_ref)}':0.0})

    grid.dae['u_ini_dict'].update({f'{str(q_s_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(q_s_ref)}':0.0})

    grid.dae['u_ini_dict'].update({f'{str(i_sa_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(i_sa_ref)}':0.0})

    grid.dae['u_ini_dict'].update({f'{str(i_sr_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(i_sr_ref)}':0.0})

    grid.dae['params_dict'].update({f'{str(v_lvrt)}':0.8})

    ### outputs
    grid.dae['h_dict'].update({f"m_ref_{name}":m_ref})
    grid.dae['h_dict'].update({f"v_sd_{name}":v_sd})
    grid.dae['h_dict'].update({f"v_sq_{name}":v_sq})
    grid.dae['h_dict'].update({f"lvrt_{name}":lvrt})


    ## VSC model 
    # m = sym.Symbol(f"m_{name}", real=True)
    # theta_t = sym.Symbol(f"theta_t_{name}", real=True)
      
    ### dynamic states
    #m_f = sym.Symbol(f"m_f_{name}", real=True)

    ### algebraic states
    i_si = sym.Symbol(f"i_si_{name}", real=True)
    i_sr = sym.Symbol(f"i_sr_{name}", real=True)       
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    q_s = sym.Symbol(f"q_s_{name}", real=True)

    ### parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    U_n = sym.Symbol(f"U_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            

    
    params_list = ['S_n','F_n','U_n','X_s','R_s']
    
    ### auxiliar
    v_si = V_s*sin(theta_s)  # v_D, e^(-j)
    v_sr = V_s*cos(theta_s)  # v_Q
    Omega_b = 2*np.pi*F_n
    m = m_ref
    theta_t = theta_t_ref
    v_t_m = m*v_dc
    v_tr = v_t_m*cos(theta_t)
    v_ti = v_t_m*sin(theta_t)
    
    ### dynamic equations            

    ### algebraic equations   
    v_ti = v_ti_ref 
    v_tr = v_tr_ref 
    g_i_si = v_ti - R_s*i_si + X_s*i_sr - v_si  
    g_i_sr = v_tr - R_s*i_sr - X_s*i_si - v_sr 
    g_p_s  = i_si*v_si + i_sr*v_sr - p_s  
    g_q_s  = i_si*v_sr - i_sr*v_si - q_s 

    ### dae 
    f_vsg = []
    x_vsg = []
    g_vsg = [g_i_si,g_i_sr,g_p_s,g_q_s]
    y_vsg = [  i_si,  i_sr,  p_s,  q_s]

    grid.dae['f'] += f_vsg
    grid.dae['x'] += x_vsg
    grid.dae['g'] += g_vsg
    grid.dae['y_ini'] += y_vsg  
    grid.dae['y_run'] += y_vsg  
    
    # grid.dae['u_ini_dict'].update({f'{m}':1.0})
    # grid.dae['u_run_dict'].update({f'{m}':1.0})

    # grid.dae['u_ini_dict'].update({f'{theta_t}':0.0})
    # grid.dae['u_run_dict'].update({f'{theta_t}':0.0})

    grid.dae['u_ini_dict'].update({f'{v_dc}':1.2})
    grid.dae['u_run_dict'].update({f'{v_dc}':1.2})

    grid.dae['xy_0_dict'].update({str(p_s):0.5})
       
    ### outputs
    grid.dae['h_dict'].update({f"{str(p_s)}":p_s})
    grid.dae['h_dict'].update({f"{str(q_s)}":q_s})
    grid.dae['h_dict'].update({f"v_ti_{name}":v_ti})
    grid.dae['h_dict'].update({f"v_tr_{name}":v_tr})    
    grid.dae['h_dict'].update({f"{str(i_si)}":i_si})
    grid.dae['h_dict'].update({f"{str(i_sr)}":i_sr})

    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]}) 

    if 'monitor' in data_dict:
        if data_dict['monitor'] == True:
            V_dc_b = U_n*np.sqrt(2)
            grid.dae['h_dict'].update({f"v_dc_v_{name}":v_dc*V_dc_b})
            grid.dae['h_dict'].update({f"v_ac_v_{name}":v_t_m*U_n})
            

    p_W   = p_s * S_n
    q_var = q_s * S_n

    return p_W,q_var

def sym2model():
    v_sd = sym.Symbol(f"v_sd", real=True)
    v_sq = sym.Symbol(f"v_sq", real=True)
    p_s_ref = sym.Symbol(f"p_s_ref", real=True)
    q_s_ref = sym.Symbol(f"q_s_ref", real=True)
    i_sa_ref = sym.Symbol(f"i_sa_ref", real=True)
    i_sr_ref = sym.Symbol(f"i_sr_ref", real=True)
    i_sd_pq_ref,i_sq_pq_ref = sym.symbols(f'i_sd_pq_ref,i_sq_pq_ref', real=True)
    i_sd_ar_ref,i_sq_ar_ref = sym.symbols(f'i_sd_ar_ref,i_sq_ar_ref', real=True)


    g_i_sd_pq_ref  = i_sd_pq_ref*v_sd + i_sq_pq_ref*v_sq - p_s_ref  
    g_i_sq_pq_ref  =-i_sq_pq_ref*v_sd + i_sd_pq_ref*v_sq - q_s_ref 

    sol = sym.solve([g_i_sd_pq_ref,g_i_sq_pq_ref],[i_sd_pq_ref,i_sq_pq_ref])

    print(sol)

    v_m = sym.sqrt(v_sd**2 + v_sq**2)
    g_i_sd_ar_ref  = i_sd_ar_ref*v_sd/v_m + i_sq_ar_ref*v_sq/v_m - i_sa_ref  
    g_i_sq_ar_ref  =-i_sq_ar_ref*v_sd/v_m + i_sd_ar_ref*v_sq/v_m - i_sr_ref 

    sol = sym.solve([g_i_sd_ar_ref,g_i_sq_ar_ref],[i_sd_ar_ref,i_sq_ar_ref])

    print(sol)

    # simplified PV module

    K_vt,K_it = sym.symbols(f"K_vt,K_it", real=True)
    V_oc,V_mp,I_sc = sym.symbols(f"V_oc,V_mp,I_sc", real=True)
    temp_k,irrad = sym.symbols(f"temp_k,irrad", real=True)
    T_stc_k,i,v = sym.symbols(f"T_stc_k,i,v", real=True)

    V_oc_t = V_oc * (1+K_vt/100.0*( temp_k - T_stc_k))
    V_mp_t = V_mp * (1+K_vt/100.0*( temp_k - T_stc_k))
    I_sc_t = I_sc*(1 + K_it/100*(temp_k - T_stc_k))
    I_mp_i = I_sc_t*irrad

    v_1,i_1 = V_mp_t,I_mp_i
    v_2,i_2 = V_oc_t,0

    # (v_1 - v)/(v_1 - v_2) = (i_1 - i)/(i_1 - i_2)
    p_mp = V_mp_t*I_mp_i
    v_dc = v_1 - (i_1 - i)*(v_1 - v_2)/(i_1 - i_2) 


sym2model()
if __name__ == "__main__":

    from pydae.bmapu import bmapu_builder
    
    data = {
    "system":{"name":"test_model","S_base":100e6, "K_p_agc":0.0,"K_i_agc":0.0,"K_xif":0.01},       
    "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
             {"name":"2", "P_W":0.0,"Q_var":0.0,"U_kV":20.0}
            ],
    "lines":[{"bus_j":"1", "bus_k":"2", "X_pu":0.05,"R_pu":0.0,"Bs_pu":0.0,"S_mva":10000.0}],
    "genapes":[{"bus":"2","S_n":10000e6,"F_n":50.0,"X_v":0.001,"R_v":0.0,"K_delta":0.001,"K_alpha":1e-6}]
    }

    grid = bmapu_builder.bmapu(data)
    #grid.checker()
    grid.uz_jacs = True
    grid.verbose = False
    grid.construct('test_model')

    bus_name = "1"
    data_dict = {"bus":bus_name,"type":"vsc_l_pq",
                 "S_n":1e6,"U_n":400.0,"F_n":50.0,
                 "X_s":0.1,"R_s":0.01,"monitor":True}
    p_W,q_var = vsc_l_pq(grid,'1','1',data_dict)

    # grid power injection
    idx_bus = [bus['name'] for bus in grid.data['buses']].index(bus_name) # get the number of the bus where the syn is connected

    S_base = sym.Symbol('S_base', real = True)
    grid.dae['g'][idx_bus*2]   += -p_W/S_base
    grid.dae['g'][idx_bus*2+1] += -q_var/S_base
    
    grid.compile()   

    import test_model

    model = test_model.model()
    model.ini({'p_s_ref_1':0.5,'q_s_ref_1':0.2,
               'v_ref_2':0.7},'xy_0.json')

    model.report_y()
    model.report_z()
    model.report_u()



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def pmsm_1(grid,name,bus_name,data_dict):
    '''

    parameters
    ----------

    S_n: nominal power in VA
    F_n: nominal frequency in Hz
    X_v: coupling reactance in pu (base machine S_n)
    R_v: coupling resistance in pu (base machine S_n)
    K_delta: if K_delta>0.0 current generator is converted to reference machine 
    K_alpha: alpha gain to obtain Domega integral 

    inputs
    ------

    alpha: RoCoF in pu if K_alpha = 1.0
    omega_ref: frequency in pu
    v_ref: internal voltage reference

    example
    -------

    "vscs": [{"type":"vsc_l","S_n":1e6,"F_n":50.0,"X_s":0.1,"R_s":0.01}]

    v_t = m*v_dc/sqrt(6)

    v_t_pu*v_b_ac = m*v_dc_pu*v_b_dc/sqrt(6)

    S_b = S_n
    U_b = U_n
    Omega_b = 2*np.pi*F_b
    Omega_rb = Omega_rn
    Tau_b = S_b/Omega_rb
    V_dc_b = sqrt(2)*U_b
    I_b = (np.sqrt(3)*U_b)
    I_dq_b = I_b*np.sqrt(2)
    V_dq_b = U_b*np.sqrt(2/3)

    V_dq_b and I_dq_b defined as below verify that: 
    3/2*I_dq_b*V_dq_b = 3/2*I_b*np.sqrt(2)*U_b*np.sqrt(2/3) = np.sqrt(3)*U_b*I_b

    Z_b = U_b**2/S_b
    Z_b = (V_dq_b/np.sqrt(2/3))**2/S_b = 3/2*V_dq_b**2/S_b
    L_m_b = Z_b/Omega_rb

    tau_r = 3/2*Phi*N_pp*i_mq

    tau_r_pu = (3/2*Phi*N_pp*i_mq_pu)*I_dq_b/Tau_b  = K_tau * i_mq_pu
    with K_tau = 3/2*Phi*N_pp*I_dq_b/Tau_b

    0 =               - R_s*i_md - omega_r*L_m*i_mq - v_md  
    0 = omega_r*Phi_m - R_s*i_mq + omega_r*L_m*i_md - v_mq 

    0 =               (- R_s*i_md_pu/V_dq_b - omega_r*L_m*i_mq_pu/V_dq_b - v_md_pu/I_dq_b)*I_dq_b*V_dq_b
    0 =               (- R_s_pu*i_md_pu/V_dq_b - omega_r*L_m_pu*i_mq_pu*Z_b/Omega_rb/(V_dq_b*Z_b) - v_md_pu/(I_dq_b*Z_b))*I_dq_b*V_dq_b*Z_b
    0 =               (- R_s_pu*i_md_pu/V_dq_b - omega_r_pu*L_m_pu*i_mq_pu/(V_dq_b) - v_md_pu/(I_dq_b*Z_b))*I_dq_b*V_dq_b*Z_b


    v_t_pu*v_b_ac = m*V_dc_b*sqrt(2)*U_b/sqrt(6)
    v_t_pu = m*V_dc_b*sqrt(2)/sqrt(6) = m*V_dc_b*sqrt(3)


    

    '''

    sin = sym.sin
    cos = sym.cos

    # inputs
    V_s = sym.Symbol(f"V_{bus_name}", real=True)
    theta_s = sym.Symbol(f"theta_{bus_name}", real=True)
    m = sym.Symbol(f"m_{name}", real=True)
    theta_t = sym.Symbol(f"theta_t_{name}", real=True)
    v_dc = sym.Symbol(f"v_dc_{name}", real=True)
    Dp_e_ref = sym.Symbol(f"Dp_e_ref_{name}", real=True)
    u_dummy = sym.Symbol(f"u_dummy_{name}", real=True)
    dnu_w_ramp_ref = sym.Symbol(f"dnu_w_ramp_ref_{name}", real=True)

    # dynamic states
    nu_w_ramp = sym.Symbol(f"nu_w_ramp_{name}", real=True)

    # algebraic states
    i_si = sym.Symbol(f"i_si_{name}", real=True)
    i_sr = sym.Symbol(f"i_sr_{name}", real=True)       
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    q_s = sym.Symbol(f"q_s_{name}", real=True)
    p_s_ref = sym.Symbol(f"p_s_ref_{name}", real=True)
    nu_w_0 = sym.Symbol(f"nu_w_0_{name}", real=True)
    i_md,i_mq,v_md,v_mq,tau_r = sym.symbols(f'i_md_{name},i_mq_{name},v_md_{name},v_mq_{name},tau_r_{name}', real=True)
    
    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            
    X_s = sym.Symbol(f"X_s_{name}", real=True)
    R_s = sym.Symbol(f"R_s_{name}", real=True)
    A_l = sym.Symbol(f"A_l_{name}", real=True)
    B_l = sym.Symbol(f"B_l_{name}", real=True)
    C_l = sym.Symbol(f"C_l_{name}", real=True)
    K_pow = sym.Symbol(f"K_pow_{name}", real=True)
    C_1,C_2,C_3 = sym.symbols(f"C_1_{name},C_2_{name},C_3_{name}", real=True)
    C_4,C_5,C_6 = sym.symbols(f"C_4_{name},C_5_{name},C_6_{name}", real=True)
    Nu_w_b,Lam_b = sym.symbols(f"Nu_w_b_{name},Lam_b_{name}", real=True)
    K_tr,D_tr,Omega_t_b = sym.symbols(f"K_tr_{name},D_tr_{name},Omega_t_b_{name}", real=True)
    C_p_b,T_beta,T_mppt,K_mppt = sym.symbols(f"C_p_b_{name},T_beta_{name},T_mppt_{name},K_mppt_{name}", real=True)
    K_p_beta = sym.Symbol(f"K_p_beta_{name}", real=True)
    K_i_beta = sym.Symbol(f"K_i_beta_{name}", real=True)
    H_t,H_r= sym.symbols(f"H_t_{name},H_r_{name}", real=True)
    K_tr,D_tr= sym.symbols(f"K_tr_{name},D_tr_{name}", real=True)
    R_m,L_m,Phi_m = sym.symbols(f'R_m_{name},L_m_{name},Phi_m_{name}', real=True)

    params_list = ['S_n','F_n','X_s','R_s']
    
    mode = ['vsc_g','v_dc']
    mode = ['aero','pitch','mech','mppt','vsc_m','v_dc','vsc_g']

    # AUXILIAR
    Omega_b = 2*np.pi*F_n
    grid.dae['params_dict'].update({f"S_n_{name}":data_dict['S_n']})

    ## Aerodinamic
    nu_w = nu_w_0 + nu_w_ramp
    theta_tr = sym.Symbol(f"theta_tr_{name}", real=True)
    omega_t  = sym.Symbol(f"omega_t_{name}", real=True)
    omega_r  = sym.Symbol(f"omega_r_{name}", real=True)
    beta     = sym.Symbol(f"beta_{name}", real=True)
    nu_w_ramp= sym.Symbol(f"nu_w_ramp_{name}", real=True) 
    p_w_mppt_lpf = sym.Symbol(f"p_w_mppt_lpf_{name}", real=True)
    lam = Lam_b*(omega_t/Omega_t_b)/(nu_w/Nu_w_b) # pu
    inv_lam_i =  1/(lam + 0.08*beta) - 0.035/(beta**3 + 1.0)   
    c_p = C_1*(C_2*inv_lam_i - C_3*beta - C_4)*sym.exp(-C_5*inv_lam_i) + C_6*lam 
    c_p_pu = c_p/C_p_b # (pu)
    p_w_nosat = K_pow*c_p_pu*(nu_w/Nu_w_b)**3 
    p_w = sym.Piecewise((0.0,p_w_nosat<0.0), (p_w_nosat,True)) 
    dnu_w_ramp = dnu_w_ramp_ref - 0.0001*nu_w_ramp

    grid.dae['u_ini_dict'].update({f'dnu_w_ramp_ref_{name}':0.0})
    grid.dae['u_run_dict'].update({f'dnu_w_ramp_ref_{name}':0.0})  
    grid.dae['u_ini_dict'].update({f'nu_w_0_{name}':10.0})
    grid.dae['u_run_dict'].update({f'nu_w_0_{name}':10.0})  
    grid.dae['f'] += [dnu_w_ramp]
    grid.dae['x'] += [ nu_w_ramp]
    grid.dae['h_dict'].update({f"nu_w_{name}":nu_w})

    
    ## MPPT
    K_mppt3  = sym.Symbol(f"K_mppt3_{name}", real=True)
    p_ref_ext = sym.Symbol(f"p_ref_ext_{name}", real=True)
    Lam_opt = Lam_b
    beta_mppt = 0.0
    nu_w_mppt = Nu_w_b*(omega_t/Omega_t_b)   
    inv_lam_i_mppt =  1/(Lam_opt) - 0.035   
    c_p_mppt = C_1*(C_2*inv_lam_i_mppt - C_4)*sym.exp(-C_5*inv_lam_i_mppt) + C_6*Lam_opt 
    c_p_mppt_pu = c_p_mppt/C_p_b # (pu)
    p_w_mppt_ref = K_pow*c_p_mppt_pu*(nu_w_mppt/Nu_w_b)**3
    omega_r_th = 0.2
    test_1 = (np.abs(Dp_e_ref)>0.01) & (omega_r>omega_r_th)
    p_w_mppt = sym.Piecewise((p_w_mppt_lpf,test_1), (p_w_mppt_ref,True))
    K_mppt = sym.Piecewise((0.0,test_1), (1.0,True))
    Dp_e = sym.Piecewise((Dp_e_ref,test_1), (Dp_e_ref * (1 + 50*(omega_r - omega_r_th)),True))
    p_w_mmpt_ref = p_w_mppt_lpf #+ Dp_e   test
    p_w_mmpt_ref = K_mppt3*omega_t**3 + R_m*(i_md**2 + i_mq**2)
    p_m_ref = sym.Piecewise((p_w_mmpt_ref+p_ref_ext,p_ref_ext<0), (p_w_mmpt_ref,True))
    dp_w_mppt_lpf  = K_mppt/T_mppt*(p_m_ref - p_w_mppt_lpf)
    if 'aero' in mode and 'mppt' in mode:
        grid.dae['f'] += [dp_w_mppt_lpf]
        grid.dae['x'] += [ p_w_mppt_lpf]
    grid.dae['params_dict'].update({f"K_mppt3_{name}":0.4})
    grid.dae['u_ini_dict'].update({f'p_ref_ext_{name}':0.0})
    grid.dae['u_run_dict'].update({f'p_ref_ext_{name}':0.0})  

    ## Pitch
    xi_beta  = sym.Symbol(f"xi_beta_{name}", real=True)
    beta_ext  = sym.Symbol(f"beta_ext_{name}", real=True)
    tau_r  = sym.Symbol(f"tau_r_{name}", real=True)
    Omega_r_max  = sym.Symbol(f"Omega_r_max_{name}", real=True)
    Omega_r_b = Omega_t_b
    epsilon_beta_nosat = omega_r - Omega_r_max
    epsilon_beta = sym.Piecewise((0.0,epsilon_beta_nosat<0),(epsilon_beta_nosat,True)) # test
    beta_ref = K_p_beta*epsilon_beta + K_i_beta*xi_beta
    p_r = tau_r*omega_r
    dxi_beta  = epsilon_beta - xi_beta*1e-8 # test
    #dxi_beta  = epsilon_beta_nosat - xi_beta*1e-8 # test
    dbeta     = 1.0/T_beta*(beta_ref + beta_ext - beta)

    grid.dae['u_ini_dict'].update({f'beta_ext_{name}':0.0})
    grid.dae['u_run_dict'].update({f'beta_ext_{name}':0.0})  
    
    grid.dae['params_dict'].update({f"Omega_r_max_{name}":1.2})

    if 'pitch' in mode:
        grid.dae['f'] += [dxi_beta,dbeta]
        grid.dae['x'] += [ xi_beta, beta]
    else:
        grid.dae['u_ini_dict'].update({f'beta_{name}':0.0})
        grid.dae['u_run_dict'].update({f'beta_{name}':0.0})          

    if not 'aero' in mode:
        p_w  = sym.Symbol(f"p_w_{name}", real=True)
        grid.dae['u_ini_dict'].update({f'p_w_{name}':0.0})
        grid.dae['u_run_dict'].update({f'p_w_{name}':0.0})   

    grid.dae['h_dict'].update({f"nu_w_{name}":nu_w})
    grid.dae['params_dict'].update({f"T_beta_{name}":2.0})
    grid.dae['params_dict'].update({f"K_p_beta_{name}":100.0})
    grid.dae['params_dict'].update({f"K_i_beta_{name}":1.0})
    grid.dae['h_dict'].update({f"p_w_{name}":p_w})

    omega_t_abs = sym.Abs(omega_t) + 0.01
    omega_r_abs = sym.Abs(omega_r) + 0.01
    
    ## Mechanical system: 2 mass equivalent    
    dtheta_tr = omega_t - omega_r - u_dummy
    domega_t  = 1.0/(2*H_t*omega_t_abs)*(p_w - K_tr*theta_tr - D_tr*(omega_t-omega_r))
    domega_r  = 1.0/(2*H_r*omega_r_abs)*(K_tr*theta_tr + D_tr*(omega_t-omega_r) - p_r)

    if 'mech' in mode:
        grid.dae['f'] += [dtheta_tr,domega_t,domega_r]
        grid.dae['x'] += [ theta_tr, omega_t, omega_r]
    else:
        grid.dae['u_ini_dict'].update({f'omega_r_{name}':1.2})
        grid.dae['u_run_dict'].update({f'omega_r_{name}':1.2})        

    p_w_0 = 0.8
    omega_t_0,omega_r_0,beta_0 = 1.2,1.2,0.0
    grid.dae['xy_0_dict'].update({str(theta_tr):p_w_0})
    grid.dae['xy_0_dict'].update({str(omega_t):omega_t_0})
    grid.dae['xy_0_dict'].update({str(omega_r):omega_r_0})
    grid.dae['xy_0_dict'].update({str(beta):beta_0})
    H_t, H_r = data_dict['H_t'],data_dict['H_r']
    grid.dae['params_dict'].update({f"H_t_{name}":H_t})
    grid.dae['params_dict'].update({f"H_r_{name}":H_r})
    w_tr = 2*np.pi*data_dict['w_tr']
    d_tr = 2*np.pi*data_dict['d_tr']
    K_tr = 2*H_r*H_t*w_tr**2/(H_r + H_t)
    D_tr =  4*np.pi*H_r*H_t*d_tr*w_tr/(H_r + H_t)
    grid.dae['params_dict'].update({f"K_tr_{name}":K_tr})
    grid.dae['params_dict'].update({f"D_tr_{name}":D_tr}) 
    beta_0 = 0.0  # initial input
    p_ref = 0.5*0.657/0.9 # initial input
    inv_lam_i =  1/(lam + 0.08*beta_0) - 0.035/(beta_0**3 + 1.0)   
    # c_p = C_1*(C_2*inv_lam_i - C_3*beta_0 - C_4)*np.exp(-C_5*inv_lam_i) + C_6*lam 
    # c_p_pu = c_p/C_p_b # (pu)
    # #p_m_ref = K_p*c_p*(nu_w/nu_w_b)**3
    # nu_w_0 = (p_ref/(K_pow*c_p_pu))**(1.0/3.0)*Nu_w_b
    # omega_t_0 = lam*(nu_w_0/Nu_w_b)/Lam_b*Omega_t_b


    ## machine + VSC control
    v_dc,p_r_ref = sym.symbols(f"v_dc_{name},p_r_ref_{name}", real=True)   
    omega_pll_f,T_pll,K_f = sym.symbols(f'omega_pll_f_{name},T_pll_{name},K_f_{name}', real=True)
    rocof,K_h = sym.symbols(f'rocof_{name},K_h_{name}', real=True)
    omega_e = omega_r # from mechanical system
    

    i_mq_ref,i_md_ref,p_ppc = sym.symbols(f'i_mq_ref_{name},i_md_ref_{name},p_ppc_{name}', real=True)
    p_f = K_f*(1.0 - omega_pll_f) 
    p_h = -K_h*rocof

    grid.dae['h_dict'].update({f"p_f_{name}":p_f})
    grid.dae['h_dict'].update({f"q_h_{name}":p_h}) 


    if 'mppt' in mode:
        # K_w_mppt = sym.Symbol(f'K_w_mppt_{name}', real=True)
        # p_m_ref = p_w_mmpt_ref # from mppt 
        # omega_r_ref = nu_w*K_w_mppt
        # grid.dae['params_dict'].update({f"K_w_mppt_{name}":8.0}) 
        p_r_ref = p_w_mppt_lpf + p_ppc + p_f + p_h
    else:
        if not 'aero' in mode:
            grid.dae['u_ini_dict'].update({f'p_w_{name}':0.0})
            grid.dae['u_run_dict'].update({f'p_w_{name}':0.0})   
        if not 'mech' in mode:
            grid.dae['u_ini_dict'].update({f'p_r_ref_{name}':0.0})
            grid.dae['u_run_dict'].update({f'p_r_ref_{name}':0.0})   
        else:
            omega_r_ref,K_omega_r = sym.symbols(f'omega_r_ref_{name},K_omega_r_{name}',real=True)
            K_w_mppt = sym.Symbol(f'K_w_mppt_{name}', real=True)
            #p_m_ref = p_w_mmpt_ref # from mppt 
            #omega_r_ref = nu_w*K_w_mppt
            grid.dae['params_dict'].update({f"K_w_mppt_{name}":8.0}) 
            #p_m_ref = K_omega_r*(omega_r_ref - omega_r)
            grid.dae['params_dict'].update({f"K_omega_r_{name}":1.0}) 
            grid.dae['u_ini_dict'].update({f'omega_r_ref_{name}':1.0})
            grid.dae['u_run_dict'].update({f'omega_r_ref_{name}':1.0})      
            grid.dae['u_ini_dict'].update({f'u_dummy_{name}':0.0})
            grid.dae['u_run_dict'].update({f'u_dummy_{name}':0.0})   

    grid.dae['params_dict'].update({f"K_f_{name}":0.0}) 
    grid.dae['params_dict'].update({f"K_h_{name}":0.0}) 

    g_i_mq_ref  = Phi_m*i_mq_ref*omega_r - p_r_ref  
    g_v_md = -L_m*i_mq_ref*omega_e - R_m*i_md_ref - v_md
    g_v_mq =  L_m*i_md_ref*omega_e + Phi_m*omega_e - R_m*i_mq_ref - v_mq

    if 'vsc_m' in mode:
        grid.dae['g'] += [g_i_mq_ref,g_v_md,g_v_mq]
        grid.dae['y_ini'] += [ i_mq_ref, v_md, v_mq]
        grid.dae['y_run'] += [ i_mq_ref, v_md, v_mq]
        grid.dae['u_ini_dict'].update({f'i_md_ref_{name}':0.0})
        grid.dae['u_run_dict'].update({f'i_md_ref_{name}':0.0})
        grid.dae['u_ini_dict'].update({f'p_ppc_{name}':0.0})
        grid.dae['u_run_dict'].update({f'p_ppc_{name}':0.0}) 
    ## machine + VSC in per unit
    i_md,i_mq,v_md,v_mq,tau_r = sym.symbols(f'i_md_{name},i_mq_{name},v_md_{name},v_mq_{name},tau_r_{name}', real=True)
    omega_e = omega_r # from mechanical system

    g_i_md = -L_m*i_mq*omega_e - R_m*i_md - v_md
    g_i_mq =  L_m*i_md*omega_e + Phi_m*omega_e - R_m*i_mq - v_mq
    g_tau_r = Phi_m*i_mq - tau_r

    grid.dae['h_dict'].update({f"p_m_{name}":i_md*v_md + i_mq*v_mq})
    grid.dae['h_dict'].update({f"q_m_{name}":i_mq*v_md - i_md*v_mq}) 


    if 'vsc_m' in mode:
        grid.dae['g'] += [g_i_md,g_i_mq,g_tau_r]
        grid.dae['y_ini'] += [i_md,i_mq,tau_r]
        grid.dae['y_run'] += [i_md,i_mq,tau_r]
        grid.dae['params_dict'].update({f"R_m_{name}":data_dict['R_m']}) 
        grid.dae['params_dict'].update({f"L_m_{name}":data_dict['L_m']}) 
        grid.dae['params_dict'].update({f"Phi_m_{name}":data_dict['Phi_m']}) 

    ## grid side VSC control  
    i_sd_ref,i_sq_ref,v_td_ref,v_tq_ref = sym.symbols(f'i_sd_ref_{name},i_sq_ref_{name},v_td_ref_{name},v_tq_ref_{name}', real=True)
    v_dc_ref,q_s_ref = sym.symbols(f'v_dc_ref_{name},q_s_ref_{name}', real=True)
    K_pdc,K_idc,K_ffdc = sym.symbols(f'K_pdc_{name},K_idc_{name},K_ffdc_{name}', real=True)
    omega_coi = sym.symbols(f'omega_coi', real=True)
    Domega_pll,theta_pll,xi_pll = sym.symbols(f'Domega_pll_{name},theta_pll_{name},xi_pll_{name}', real=True)
    K_p_pll,K_i_pll,K_d_pll = sym.symbols(f'K_p_pll_{name},K_i_pll_{name},K_d_pll_{name}', real=True)
    # omega_vsg,theta_vsg = sym.symbols(f'omega_vsg_{name},theta_vsg_{name}', real=True)
    # K_p_vsg,K_i_vsg = sym.symbols(f'K_p_vsg_{name},K_i_vsg_{name}', real=True)
    # X_vsg,v_vsg = sym.symbols(f'X_vsg_{name},v_vsg_{name}', real=True)
    delta = theta_s # ideal PLL
    v_sD = V_s*sin(theta_s)  # v_si   e^(-j)
    v_sQ = V_s*cos(theta_s)  # v_sr
    v_sd = v_sD * cos(delta) - v_sQ * sin(delta)   
    v_sq = v_sD * sin(delta) + v_sQ * cos(delta)
    
    omega_s = omega_coi
    
    v_tD_ref = v_td_ref * cos(delta) + v_tq_ref * sin(delta)   
    v_tQ_ref =-v_td_ref * sin(delta) + v_tq_ref * cos(delta)    
    v_ti_ref = v_tD_ref
    v_tr_ref = v_tQ_ref   
    m_ref = sym.sqrt(v_tr_ref**2 + v_ti_ref**2)/v_dc
    theta_t_ref = sym.atan2(v_ti_ref,v_tr_ref) 
    
    v_sd_pll = v_sD * cos(theta_pll) - v_sQ * sin(theta_pll)   
    Domega_pll = K_p_pll*v_sd_pll + K_i_pll*xi_pll 
    omega_pll = Domega_pll + 1.0
    dtheta_pll = 2*np.pi*50*(omega_pll - omega_coi) 
    dxi_pll = v_sd_pll 
    domega_pll_f = 1/T_pll*(omega_pll - omega_pll_f)
    

    eq_p_s_ref = -p_s_ref - K_pdc*(v_dc_ref - v_dc) +  K_ffdc*(i_md*v_md + i_mq*v_mq)
    eq_i_sd_ref  = i_sd_ref*v_sd + i_sq_ref*v_sq - p_s_ref  
    eq_i_sq_ref  = i_sq_ref*v_sd - i_sd_ref*v_sq - q_s_ref
    eq_v_td_ref  = v_td_ref - R_s*i_sd_ref - X_s*i_sq_ref - v_sd  
    eq_v_tq_ref  = v_tq_ref - R_s*i_sq_ref + X_s*i_sd_ref - v_sq 
    eq_rocof = rocof - domega_pll_f

    if 'vsc_g' in mode:
        grid.dae['f'] += [dtheta_pll,dxi_pll,domega_pll_f]
        grid.dae['x'] += [ theta_pll, xi_pll, omega_pll_f]

        grid.dae['g'] += [eq_p_s_ref,eq_i_sd_ref,eq_i_sq_ref,eq_v_td_ref,eq_v_tq_ref,eq_rocof]
        grid.dae['y_ini'] += [p_s_ref, i_sd_ref, i_sq_ref, v_td_ref, v_tq_ref,rocof]  
        grid.dae['y_run'] += [p_s_ref, i_sd_ref, i_sq_ref, v_td_ref, v_tq_ref,rocof] 
        grid.dae['u_ini_dict'].update({f'v_dc_ref_{name}':1.5})
        grid.dae['u_run_dict'].update({f'v_dc_ref_{name}':1.5})
        grid.dae['u_ini_dict'].update({f'q_s_ref_{name}':0.0})
        grid.dae['u_run_dict'].update({f'q_s_ref_{name}':0.0})
        grid.dae['params_dict'].update({f'K_pdc_{name}':data_dict['K_pdc']}) 
        grid.dae['params_dict'].update({f'K_ffdc_{name}':1.0}) 
        grid.dae['params_dict'].update({f'K_p_pll_{name}':10,f'K_i_pll_{name}':100}) 
        grid.dae['params_dict'].update({f'T_pll_{name}':0.1}) 
        grid.dae['xy_0_dict'].update({str(i_sr):0.1})
        grid.dae['h_dict'].update({f"omega_pll_{name}":omega_pll})
        grid.dae['h_dict'].update({f"omega_pll_f_{name}":omega_pll_f})
        grid.dae['h_dict'].update({f"rocof_{name}":rocof})

    ## grid side VSC
    v_sr =  V_s*cos(theta_s)  # v_Q
    v_si =  V_s*sin(theta_s)  # v_D, e^(-j)
    i_tr = i_sr
    i_ti = i_si
    v_tr = v_tr_ref
    v_ti = v_ti_ref
    i_tdc = (i_ti*v_ti + i_tr*v_tr)/v_dc
    g_i_si = v_ti - R_s*i_si - X_s*i_sr - v_si  
    g_i_sr = v_tr - R_s*i_sr + X_s*i_si - v_sr 
    g_p_s  =  i_sr*v_sr + i_si*v_si - p_s  
    g_q_s  =  i_sr*v_si - i_si*v_sr - q_s 

    if 'vsc_g' in mode:

        grid.dae['g'] += [g_i_si,g_i_sr,g_p_s,g_q_s]
        grid.dae['y_ini'] += [  i_si,  i_sr,  p_s,  q_s]  
        grid.dae['y_run'] += [  i_si,  i_sr,  p_s,  q_s]  
        
        grid.dae['u_ini_dict'].update({f'v_dc_ref_{name}':1.2})
        grid.dae['u_run_dict'].update({f'v_dc_ref_{name}':1.2})

        grid.dae['u_ini_dict'].update({f'{Dp_e_ref}':0.0})
        grid.dae['u_run_dict'].update({f'{Dp_e_ref}':0.0})

        grid.dae['u_ini_dict'].update({f'{nu_w}':10.0})
        grid.dae['u_run_dict'].update({f'{nu_w}':10.0})

        grid.dae['u_ini_dict'].update({f'{u_dummy}':0.0})
        grid.dae['u_run_dict'].update({f'{u_dummy}':0.0})

        grid.dae['params_dict'].update({f"R_s_{name}":data_dict['R_s']}) 
        grid.dae['params_dict'].update({f"X_s_{name}":data_dict['X_s']}) 

        grid.dae['h_dict'].update({f"{str(p_s)}":p_s})
        grid.dae['h_dict'].update({f"{str(q_s)}":q_s}) 
        grid.dae['h_dict'].update({f"{str(i_si)}":i_si})
        grid.dae['h_dict'].update({f"{str(i_sr)}":i_sr})
        grid.dae['h_dict'].update({f"i_tdc_{name}":i_tdc})

    ## DC link   
    C_dc,i_dc = sym.symbols(f'C_dc_{name},i_dc_{name}', real=True)
    i_mdc = (i_md*v_md + i_mq*v_mq)/(v_dc + 1e-6)
    i_tdc = (i_ti*v_ti + i_tr*v_tr)/(v_dc + 1e-6)

    if 'vsc_m' in mode: 
        dv_dc = 0.5*(i_mdc - i_tdc)/(C_dc)
        grid.dae['h_dict'].update({f"i_mdc_{name}":i_mdc})
    if 'vsc_m' in mode and 'vsc_g' in mode: 
        dv_dc = 0.5*(i_mdc - i_tdc)/(C_dc)

    if not 'vsc_m' in mode and 'vsc_g' in mode: 
        dv_dc = 0.5*(i_dc - i_tdc)/(C_dc)
        grid.dae['u_ini_dict'].update({f'i_dc_{name}':0.0})
        grid.dae['u_run_dict'].update({f'i_dc_{name}':0.0})

    if 'v_dc' in mode:

        grid.dae['f'] += [dv_dc]
        grid.dae['x'] += [ v_dc]
        grid.dae['params_dict'].update({f"C_dc_{name}":data_dict['C_dc']}) 
        grid.dae['xy_0_dict'].update({str(v_dc):1.5})

    else:
        grid.dae['u_ini_dict'].update({f'v_dc_{name}':1.5})
        grid.dae['u_run_dict'].update({f'v_dc_{name}':1.5})



    # outputs


    # grid.dae['h_dict'].update({f"p_ac_{name}":p_ac})
    # grid.dae['h_dict'].update({f"p_dc_{name}":p_dc})
    # grid.dae['h_dict'].update({f"i_d_{name}":i_d})
    # grid.dae['h_dict'].update({f"u_dummy_{name}":u_dummy})
    # grid.dae['h_dict'].update({f"p_m_mppt_lpf_{name}":p_m_mppt_lpf})
    # grid.dae['h_dict'].update({f"p_t_{name}":p_m})
    # grid.dae['h_dict'].update({f"K_mppt_{name}":K_mppt})
    # grid.dae['h_dict'].update({f"lam_{name}":lam})
    # grid.dae['h_dict'].update({f"c_p_{name}":c_p})
    # grid.dae['h_dict'].update({f"c_p_pu_{name}":c_p_pu})
    # grid.dae['h_dict'].update({f"nu_w_mppt_{name}":nu_w_mppt})
    # grid.dae['h_dict'].update({f"p_m_{name}":p_m})
    

    # for item in params_list:       
    #     grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]}) 

    if 'A_l' in data_dict:
        grid.dae['params_dict'].update({f"A_l_{name}":data_dict['A_l']}) 
        grid.dae['params_dict'].update({f"B_l_{name}":data_dict['B_l']}) 
        grid.dae['params_dict'].update({f"C_l_{name}":data_dict['C_l']}) 
    else:
        grid.dae['params_dict'].update({f"A_l_{name}":0.005}) 
        grid.dae['params_dict'].update({f"B_l_{name}":0.005}) 
        grid.dae['params_dict'].update({f"C_l_{name}":0.005})         

    C_1 = 0.5176
    C_2 = 116
    C_3 = 0.4
    C_4 = 5.0
    C_5 = 21 
    C_6 = 0.0068
    Nu_w_b = 12
    Lam_b = 8.1
    Omega_t_b = 1.2
    K_pow = 1.0
    grid.dae['params_dict'].update({f"C_1_{name}":C_1})
    grid.dae['params_dict'].update({f"C_2_{name}":C_2})
    grid.dae['params_dict'].update({f"C_3_{name}":C_3})
    grid.dae['params_dict'].update({f"C_4_{name}":C_4})
    grid.dae['params_dict'].update({f"C_5_{name}":C_5})
    grid.dae['params_dict'].update({f"C_6_{name}":C_6})
    grid.dae['params_dict'].update({f"Nu_w_b_{name}":Nu_w_b})
    grid.dae['params_dict'].update({f"Lam_b_{name}":Lam_b})
    grid.dae['params_dict'].update({f"Omega_t_b_{name}":Omega_t_b})
    grid.dae['params_dict'].update({f"K_pow_{name}":K_pow})
    ## 2 mass mechanical model

    ## c_p_b:
    beta_b = 0.0
    lam = Lam_b
    inv_lam_i =  1/(lam + 0.08*beta_b) - 0.035/(beta_b**3 + 1.0)   
    C_p_b = C_1*(C_2*inv_lam_i - C_3*beta_b - C_4)*np.exp(-C_5*inv_lam_i) + C_6*lam 
    grid.dae['params_dict'].update({f"C_p_b_{name}":C_p_b})


    grid.dae['params_dict'].update({f"T_mppt_{name}":5})

    if 'vsc_g' in mode:
        p_W   = p_s * S_n
        q_var = q_s * S_n
    else:
        p_W   = 0
        q_var = 0      

    return p_W,q_var


def symmodel():

    S_b,U_b,F_b,Omega_eb,N_pp = sym.symbols('S_b,U_b,F_b,Omega_eb,N_pp', real = True)
    i_md_pu,i_mq_pu,v_md_pu,v_mq_pu,omega_e_pu = sym.symbols(f'i_md_pu,i_mq_pu,v_md_pu,v_mq_pu,omega_e_pu', real = True)
    R_m_pu,L_m_pu,Phi_m_pu,tau_r_pu,p_m_pu,q_m_pu= sym.symbols(f'R_m_pu,L_m_pu,Phi_m_pu,tau_r_pu,p_m_pu,q_m_pu', real = True)
    Omega_rb = Omega_eb/N_pp
    Tau_b = S_b/Omega_rb
    I_b = S_b/(sym.sqrt(3)*U_b)
    I_dq_b = I_b*sym.sqrt(2)
    V_dq_b = U_b*sym.sqrt(2)/sym.sqrt(3)
    V_dc_b = sym.sqrt(2)*U_b
    I_dc_b = S_b/V_dc_b

    # V_dq_b and I_dq_b defined as below verify that: 
    # 3/2*I_dq_b*V_dq_b = 3/2*I_b*sym.sqrt(2)*U_b*sym.sqrt(2/3) = np.sqrt(3)*U_b*I_b

    Z_b = U_b**2/S_b
    #Z_b = (V_dq_b/sym.sqrt(2/3))**2/S_b #= 3/2*V_dq_b**2/S_b
    L_m_b = Z_b/Omega_eb
    Phi_b = V_dq_b/Omega_eb

    #tau_r = 3/2*Phi*N_pp*i_mq

    #tau_r_pu = (3/2*Phi*N_pp*i_mq_pu)*I_dq_b/Tau_b  = K_tau * i_mq_pu
    #with K_tau = 3/2*Phi*N_pp*I_dq_b/Tau_b

    # 0 =               - R_s*i_md - omega_r*L_m*i_mq - v_md  
    # 0 = omega_r*Phi_m - R_s*i_mq + omega_r*L_m*i_md - v_mq 

    R_m = R_m_pu*Z_b
    i_md = i_md_pu*I_dq_b
    i_mq = i_mq_pu*I_dq_b
    v_md = v_md_pu*V_dq_b
    v_mq = v_mq_pu*V_dq_b
    L_m = L_m_pu*L_m_b
    omega_e = omega_e_pu*Omega_eb
    Phi_m = Phi_m_pu*Phi_b

    # PMSM electric equations
    eq_i_md =  - R_m*i_md - omega_e*L_m*i_mq - v_md  
    eq_i_mq = omega_e*Phi_m - R_m*i_mq + omega_e*L_m*i_md - v_mq 

    print(f"eq_i_md = {str(sym.simplify(eq_i_md/V_dq_b)).replace('_pu','')}")
    print(f"eq_i_mq = {str(sym.simplify(eq_i_mq/V_dq_b)).replace('_pu','')}")

    # Torque equations
    tau_r = tau_r_pu*Tau_b
    eq_tau_r = -tau_r + 3/2*Phi_m*N_pp*i_mq
    print(f"eq_tau_r = {str(sym.simplify(eq_tau_r/Tau_b)).replace('_pu','').replace('1.0*','')}")


    # PMSM Power equations
    p_m = p_m_pu*S_b
    q_m = q_m_pu*S_b

    eq_p_m = -p_m + 3/2*(v_md*i_md + v_mq*i_mq)
    eq_q_m = -q_m + 3/2*(v_md*i_mq - v_mq*i_md)

    print(f"eq_p_m = {str(sym.simplify(eq_p_m/S_b)).replace('_pu','').replace('1.0*','')}")
    print(f"eq_q_m = {str(sym.simplify(eq_q_m/S_b)).replace('_pu','').replace('1.0*','')}")

    # DC link
    v_dc_pu,C_dc_pu = sym.symbols('v_dc_pu,C_dc_pu', real=True)
    v_dc = v_dc_pu*V_dc_b
    p_mac = 3/2*(v_md*i_md + v_mq*i_mq)
    p_mdc = p_mac
    i_mdc = p_mdc/v_dc

    print(f"i_mdc = {str(sym.simplify(i_mdc/I_dc_b)).replace('_pu','').replace('1.0*','')}")

    v_tr_pu,v_ti_pu,i_tr_pu,i_ti_pu = sym.symbols('v_tr_pu,v_ti_pu,i_tr_pu,i_ti_pu', real=True)
    v_tr = v_tr_pu*V_dq_b
    v_ti = v_ti_pu*V_dq_b
    i_tr = i_tr_pu*I_dq_b
    i_ti = i_ti_pu*I_dq_b

    p_tac = 3/2*(v_tr*i_tr + v_ti*i_ti)
    p_tdc = p_tac
    i_tdc = p_tdc/v_dc
    
    print(f"i_tdc = {str(sym.simplify(i_tdc/I_dc_b)).replace('_pu','').replace('1.0*','')}")

    C_dc_b = 1/(Z_b)
    C_dc = C_dc_pu*C_dc_b

    dv_dc = 1/C_dc*(i_mdc - i_tdc)
    print(f"dv_dc = {str(sym.simplify(dv_dc/V_dc_b))}")





def mppt_omega_p(omega_t):
    
    C_1 = 0.5176
    C_2 = 116
    C_3 = 0.4
    C_4 = 5.0
    C_5 = 21 
    C_6 = 0.0068
    Nu_w_b = 12
    Lam_b = 8.1
    Omega_t_b = 1.2
    K_pow = 0.73

    ## MPPT
    Lam_opt = Lam_b
    beta_mppt = 0.0
    nu_w_mppt = Nu_w_b*(omega_t/Omega_t_b)   
    inv_lam_i_mppt =  1/(Lam_opt) - 0.035   
    c_p_mppt = C_1*(C_2*inv_lam_i_mppt - C_4)*sym.exp(-C_5*inv_lam_i_mppt) + C_6*Lam_opt 
    c_p_mppt_pu = c_p_mppt/C_p_b # (pu)
    p_w_mppt_ref = K_pow*c_p_mppt_pu*(nu_w_mppt/Nu_w_b)**3
    omega_r_th = 0.2
    # test_1 = (np.abs(Dp_e_ref)>0.01) & (omega_r>omega_r_th)
    # p_w_mppt = sym.Piecewise((p_w_mppt_lpf,test_1), (p_w_mppt_ref,True))
    # K_mppt = sym.Piecewise((0.0,test_1), (1.0,True))
    # Dp_e = sym.Piecewise((Dp_e_ref,test_1), (Dp_e_ref * (1 + 50*(omega_r - omega_r_th)),True))
    # p_w_mmpt_ref = p_w_mppt_lpf #+ Dp_e   test

    return p_w_mppt_ref


if __name__ == '__main__':
    #symmodel()

    from pydae.bmapu import bmapu_builder
    import sympy as sym

    data = {
        "system":{"name":"wecs_pmsm_test","S_base":6e6, "K_p_agc":0.0,"K_i_agc":0.0,"K_xif":0.01},
        "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":0.69},
                    {"name":"2", "P_W":0.0,"Q_var":0.0,"U_kV":0.69}
                ],
        "lines":[{"bus_j":"1", "bus_k":"2", "X_pu":0.05,"R_pu":0.01,"Bs_pu":1e-6,"S_mva":100}],
        "wecs":[
            {"type":"pmsm_1","bus":"1","S_n":6e6,
                "H_t":3.0769,"H_r":1.0, "w_tr":1.1519, "d_tr":0.01,
                "R_m":0.050378,"L_m":0.1,"Phi_m":2.7171,
                "R_s":0.01,"X_s":0.033,
                "K_pdc":4.286e4,"C_dc":10}],
        "genapes":[{"bus":"2","S_n":1e9,"F_n":50.0,"X_v":0.001,"R_v":0.0,"K_delta":0.001,"K_alpha":1e-6}]
        }


    grid = bmapu_builder.bmapu(data)
    #grid.checker()
    grid.uz_jacs = True
    grid.verbose = False
    grid.build('pmsm_test')

    import pmsm_test

    model = pmsm_test.model()
    
    
    # N = model.jac_ini.shape[0]
    # for it in range(N):
    #     print(it,model.jac_ini[it,:])

    nu_w =11.6
    omega_0 = np.clip(nu_w/13,0.4,1.2)
    xy_0 = {
        "V_1": 1.0,
        "theta_1": 0.0,
        "V_2": 1.0,
        "theta_2": 0.0,
        "omega_coi": 1.0,
        "omega_2": 1.0,
        "theta_tr_1": 0.0,
        "omega_t_1": omega_0,
        "omega_r_1": omega_0,
        "xi_beta_1":1,
        "beta_1":0.0,
        "i_sr_1": 1,
        "v_dc_1": 1.5,
        "p_w_mppt_lpf_1":1.0,
        "v_mq_1":1.0,
        "v_tq_ref_1":1.0
    }


    params = {"nu_w_1":nu_w,"Nu_w_b_1":13,"Omega_t_b_1":1.0,"Omega_r_max_1":1.0,"K_mppt3_1":1,"T_mppt_1":10,
            "K_p_beta_1":1,"K_i_beta_1":1.0,"T_beta_1":1.2,"p_ref_ext_1":0.0, "K_pdc_1":4.286e4, "C_dc_1":10,
            "v_dc_ref_1":1.4, "K_ffdc_1":0, "R_s_1":0.01, "R_m_1":0.004}


    model.ini(params,xy_0)

    model.report_x()


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""


import sympy as sym


def kundur_tgr(dae,syn_data,name,bus_name):
    '''
        "avr":{"type":"kundur_tgr","K_a":200,"T_r":0.01,"T_a":1,"T_b":10,"v_ref":1.0}
       
     '''

    avr_data = syn_data['avr']
    
    v_c = sym.Symbol(f"V_{name}", real=True)   
    v_r = sym.Symbol(f"v_r_{name}", real=True)  
    x_ab = sym.Symbol(f"x_ab_{name}", real=True)  
    xi_v  = sym.Symbol(f"xi_v_{name}", real=True)
    v_f = sym.Symbol(f"v_f_{name}", real=True)  
    T_r = sym.Symbol(f"T_r_{name}", real=True) 
    T_a = sym.Symbol(f"T_a_{name}", real=True) 
    T_b = sym.Symbol(f"T_b_{name}", real=True) 
    K_a = sym.Symbol(f"K_a_{name}", real=True)
    K_ai = sym.Symbol(f"K_ai_{name}", real=True)
    
    # inputs
    v_ref = sym.Symbol(f"v_ref_{name}", real=True) 
    v_s   = sym.Symbol(f"v_pss_{name}", real=True) 

    # auxiliar
    epsilon_v = v_ref - v_r 
    v_i = K_ai*xi_v
    u_ab = K_a * (epsilon_v + v_s + v_i)
    z_ab = (u_ab - x_ab)*T_a/T_b + x_ab
    
        
    # differential equations
    dv_r =   (v_c - v_r)/T_r
    dx_ab =  (u_ab - x_ab)/T_b      # lead compensator state
    dxi_v =   epsilon_v 

    # algebraic equations   
    g_v_f  =   z_ab - v_f 
  
    dae['f'] += [dv_r,dx_ab,dxi_v]
    dae['x'] += [ v_r, x_ab, xi_v]
    dae['g'] += [g_v_f]
    dae['y_ini'] += [v_f] 
    dae['y_run'] += [v_f]  

    dae['params_dict'].update({str(K_a):avr_data['K_a']})
    dae['params_dict'].update({str(K_ai):1e-6})
    dae['params_dict'].update({str(T_r):avr_data['T_r']})  
    dae['params_dict'].update({str(T_a):avr_data['T_a']}) 
    dae['params_dict'].update({str(T_b):avr_data['T_b']}) 

    dae['u_ini_dict'].update({str(v_ref):avr_data['v_ref']})
    dae['u_ini_dict'].update({str(v_s):0.0})

    dae['u_run_dict'].update({str(v_ref):avr_data['v_ref']})
    dae['u_run_dict'].update({str(v_s):0.0})

    dae['xy_0_dict'].update({str(v_r):1.0})
    dae['xy_0_dict'].update({str(x_ab):1.0})
    dae['xy_0_dict'].update({str(xi_v):0.0})
    
    dae['h_dict'].update({str(v_ref):v_ref})
    dae['h_dict'].update({str(v_s):v_s})